from pydantic import BaseModel
from .. import minio 

class Minio(BaseModel):
    host: str
    port: int
    bucket: str
    access_key: str
    secret_key: str

    def connect(self):
        # 初始化minio对象
        obj = minio.PyMinio(
            self.host,
            self.port,
            self.access_key,
            self.secret_key,
        )
        obj.set_bucket(self.bucket)
        return obj


from typing import Any
from pydantic import BaseModel
from redis import StrictRedis


class Redis(BaseModel):
    host: str
    port: int = 6379
    db: int = 0

    redis: Any = None
    cache: Any = None

    def get_dsn(self):
        return 'redis://{}:{}/{}'.format(self.host, self.port, self.db)

    def connect(self) -> StrictRedis:
        return StrictRedis(host=self.host, port=self.port, db=self.db, decode_responses=True)

    def instance(self) -> StrictRedis:
        if self.redis==None:
            self.redis = self.connect()
            return self.redis
        else:
            try:
                self.redis.ping()
                return self.redis
            except Exception as e:
                self.redis = self.connect()
                return self.redis

    def llen(self, queue):
        redis = self.redis or self.connect()

        try:
            return redis.llen(queue)
        except Exception:
            return self.connect().llen(queue)

    def lpush(self, queue, value):
        redis = self.redis or self.connect()

        try:
            return redis.lpush(queue, value)
        except Exception:
            return self.connect().lpush(queue, value)

    def rpop(self, queue):
        redis = self.redis or self.connect()

        try:
            return redis.rpop(queue)
        except Exception:
            return self.connect().rpop(queue)

    def cache_it(self, *, limit=100000, expire=3600, hashkeys=True, namespace="default"):
        if self.cache==None:
            from redis_cache import SimpleCache,cache_it
            my_cache = SimpleCache(limit=limit, expire=expire, hashkeys=hashkeys, host=self.host, port=self.port, db=self.db, namespace=namespace)
            self.cache = cache_it(cache=my_cache)
        return self.cache




from openpyxl import Workbook

class Excel:
    @classmethod
    def save_excel(cls, filename, data):
        # 实例化
        wb = Workbook()
        # 激活 worksheet
        ws = wb.active
        ws.title = "sheet1"

        i = 1
        for row in data:
            column=0

            if type(row)==dict:
                array = row.values()
            else:
                array = row

            for value in array:
                ws[chr(65+column)+str(i)] = value
                column += 1

            i += 1

        wb.save(filename)


from datetime import datetime, timedelta

class Datetime:
    def __init__(self, *, timestamp=-1):
        if timestamp<0:
            self._dt = datetime.now()
        else:
            self._dt = datetime.fromtimestamp(timestamp)

    def __str__(self):
        return self._dt.strftime('%Y-%m-%d %H:%M:%S')

    def change(self, **kwargs):
        self._dt = self._dt + timedelta(**kwargs)
        return self

    def timestamp(self, timestamp=-1):
        if timestamp>0:
            self._dt = datetime.fromtimestamp(timestamp)
            return self
        else:
            return int(self._dt.timestamp())


from pydantic import BaseModel
from loguru import logger

class Logger(BaseModel):
    level: str = 'INFO'
    retention: str = '7 days'
    compression: str = 'zip'
    rotation: str = '00:00'
    log_path: str = "/var/log"

    def initial(self, filename):
        filename = '{}/{}'.format(self.log_path, filename)
        logger.add(filename, rotation=self.rotation, retention=self.retention, compression=self.compression, level=self.level)

        return logger





from http.server import HTTPServer, BaseHTTPRequestHandler
import json
import functools
import requests

store = []
routers = {}

def route(path):
    def decorator(function):
        routers[path] = function

        @functools.wraps(function)
        def wrapper(*args, **kwargs):
            #print(function)
            return function(*args, **kwargs)

        return wrapper

    return decorator   

class BaseMock(BaseHTTPRequestHandler):
    def do_GET(self):
        self.send_response(200)
        self.end_headers()
        self.wfile.write(bytes(json.dumps(store), encoding='utf-8'))

        while len(store)>0:
            store.pop()

    def do_POST(self):
        path = self.path
        body = self.rfile.read(int(self.headers['content-length']))
        body = json.loads(str(body, encoding='utf-8'))

        store.append({
            "path": path,
            "body": body,
        })
        #print(path, body)

        if path in routers:
            text = routers.get(path)(self, body)

            self.send_response(200)
            self.end_headers()
            self.wfile.write(bytes(json.dumps(text), encoding='utf-8'))
        else:
            self.send_response(404)
            self.end_headers()
            self.wfile.write(b'404')


class HTTP:
    def __init__(self, host='127.0.0.1', port=80):
        self.host = host
        self.port = port

    def run(self, handler):
        server = HTTPServer((self.host,self.port), handler)
        server.serve_forever()  


class Result:
    def __init__(self, path, body):
        self.path = path
        self.body = body

class HttpMock:
    def __init__(self, *, host="127.0.0.1", port=80, path="/"):
        self.host = host
        self.port = port
        self.path = path

    def get_result(self) -> Result:
        return self._do_request()

    def clear(self):
        self._do_request()

    def _do_request(self):
        url = "http://{}:{}{}".format(self.host, self.port, self.path)
        response = requests.get(url)
        #print(response.content)
        data = json.loads(response.content)

        return [Result(item.get('path'), item.get('body')) for item in data]



import functools

def cache():
    results = {}
    
    def cached(function):
        @functools.wraps(function)
        def _cached(*args, **kwargs):
            if function not in results:
                results[function] = function(*args, **kwargs)

            return results[function]
 
        return _cached
 
    return cached

singleton = cache()



import nsq
import requests
from .mq import Writer, Reader

# NSQ 的api支持的不好，暂时使用http接口，性能不好
class NsqWriter(Writer):
    def __init__(self, *, addrs, topic):
        
        self.url = "http://{}/pub?topic={}".format(
            addrs[0], 
            topic
        )

    def write(self, message, *, callback=None):
        requests.post(self.url , data=message)

class NsqReader(Reader):
    def run(self, callback):
        def _mission(message):
            if message.body:
                is_finish = callback(message.body, message=message)
                if is_finish==True:
                    message.finish()
            else:
                message.finish()

            return True

        nsq.Reader(
            message_handler=_mission,
            **self.options
        )

        nsq.run()      


from minio import Minio
import os
import io
import base64
from io import BytesIO
import hashlib

class PyMinio:
    def __init__(self, host, port, access_key, secret_key, secure=False):
        self.client = Minio(
            "{}:{}".format(host, port),
            access_key=access_key,
            secret_key=secret_key,
            secure=secure
        )

    def set_bucket(self, bucket):
        self.bucket = bucket

    def _trim_bucket(self, object_name:str, *, trim:bool) -> str:
        if object_name[0]=='/':
            object_name = object_name[1:]
            
        # 去掉前面的桶的名字
        if trim==True:
            arr = object_name.split('/')[1:]
            object_name = os.path.join(*arr)

        return object_name

    def download(self, object_name:str, local_name:str, *, trim:bool=False) -> bool:
        object_name = self._trim_bucket(object_name, trim=trim)

        if self.exists(object_name):
            self.client.fget_object(self.bucket, object_name, local_name)
            return True
        else:
            return False

    def upload(self, file_path:str, object_name:str) -> bool:
        if self.client.bucket_exists(self.bucket) == False:
            self.client.make_bucket(self.bucket)  # 生成一个bucket，类似文件夹

        self.client.fput_object(
            bucket_name=self.bucket,
            object_name=object_name,
            file_path=file_path
        )
        return True

    def write(self, object_name, content:bytes):
        self.client.put_object(self.bucket, object_name, io.BytesIO(content), len(content))

    def exists(self, object_name:str, *, trim:bool=False) -> bool:
        object_name = self._trim_bucket(object_name, trim=trim)

        try:
            response = self.client.get_object(self.bucket, object_name)
            response.close()
            response.release_conn()
            return True
        except Exception as e:
            return False

    # 删除对象
    def remove_file(self, object_name:str, *, trim:bool=False) -> bool:
        object_name = self._trim_bucket(object_name, trim=trim)
        try:
            self.client.remove_object(self.bucket, object_name)
            #print("Sussess")
            return True
        except Exception as err:
            #print(err)
            return False

    def get_client(self) -> Minio:
        return self.client

    def upload_image(self, image_base64:str, *, object_name=None, filetype=None, bucket=None):
        partition = image_base64.find(",")
        if partition > 0:
            image_base64 = image_base64[partition:]
        else:
            image_base64 = image_base64
        image_data = base64.b64decode(image_base64)

        if object_name!=None and object_name[0]=='/':
            object_name = object_name[1:]

        if bucket==None:
            bucket = self.bucket

        #
        if object_name==None:
            if filetype==None:
                import imghdr

                # 通过二进制内容获取图片格式
                filetype = imghdr.what(None, image_data)
                if filetype==None:
                    filetype = "unknown"

            hash_md5 = hashlib.md5()
            hash_md5.update(image_data)
            image_name = hash_md5.hexdigest() + "." + filetype
            object_name = "{}/{}.{}".format(image_name[:2], image_name[2:], filetype)

        result = self.client.put_object(bucket, object_name, BytesIO(image_data), len(image_data))

        if result!=None:
            return "{}/{}".format(bucket, object_name)
        else:
            return None
        



import pymysql
from pymysql.cursors import DictCursor

class Mysql:
    def __init__(self, *, host, port, db, user, password, charset='utf8'):
        self._connect = pymysql.connect( host=host, port=port, db=db, user=user, password=password, charset=charset )
        self._cursor = self._connect.cursor(DictCursor)

    def ping(self):
        self._connect.ping(reconnect=True)

    def fetchall(self, sql, args=None):
        self.ping()
        self._cursor.execute(sql, args)
        self._connect.commit()
        return self._cursor.fetchall()

    def execute(self, sql, args=None):
        """ update, insert, delete """
        self.ping()
        self._cursor.execute(sql, args)
        self._connect.commit()

    def insert(self, sql, args=None):
        self.ping()
        self._cursor.execute(sql, args)
        id = self._cursor.lastrowid
        self._connect.commit()

        return id

    def __del__(self):
        self._connect.close()



from telnetlib import Telnet
import time

def is_alive(host, port):
    try:
        tn = Telnet()
        tn.open(host, port)
        return True
    except ConnectionRefusedError as e:
        return False

def wait_for_ready(host, port):
    for i in range(10):
        if is_alive(host, port):
            time.sleep(1)
            break
        else:
            time.sleep(1)


import threading
from multiprocessing import Process
from .network import is_alive, wait_for_ready
import time
import os
import subprocess
import signal

def start_fastapi(entry="index:app", *, host='0.0.0.0', port=8000) -> Process:
    def start_server():
        import uvicorn
        uvicorn.run(entry, host=host, port=port)

    service = Process(target=start_server, args=())
    service.start()
    
    wait_for_ready(host, port)
    return service

class Service:
    def __init__(self, command=["python", "/app/index.py"]):
        self._process = subprocess.Popen(command)

    def wait_for_ready(self, host='0.0.0.0', port=80):
        wait_for_ready(host, port)
        return self

    def terminate(self):
        self._process.send_signal(signal.SIGTERM)
        self._process.wait(20)

